﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityUtils;

[RequireComponent(typeof(CharacterController))]
[RequireComponent(typeof(Animator))]
public class SheepController : MonoBehaviour
{
    [SerializeField] 
    [Range(1, 10)]
    private float proximityRange = 1;

    [SerializeField] 
    [Range(1, 10)]
    private float speed = 10;

    [SerializeField] 
    private int maxHealth = 5;

    [SerializeField] 
    private HealthBar healthBar;
    
    [SerializeField]
    private Animator sheepAnimator;

    [SerializeField]
    private AudioClip[] sheepClips;

    [SerializeField]
    private float minIntervalBetweenSheep = 5;
    
    //HP
    private int currentHealth;
    
    //sound
    private float latestSheepSoundTime;

    //movement
    private Vector3 targetPosition;
    private Vector3 intendedMovement;
    private Vector3 movementRotation;

    //cached components
    private CharacterController characterController;
    private AudioSource audioSource;

    //static instance
    public static SheepController Instance => instance;
    private static SheepController instance;
    
    private void Awake()
    {
        if (instance != null && instance != this)
        {
            Destroy(gameObject);
            return;
        }

        instance = this;

        audioSource = GetComponent<AudioSource>();
        characterController = GetComponent<CharacterController>();
        characterController.SimpleMove(Vector3.zero);
        
        targetPosition = transform.position;
        currentHealth = maxHealth;
        healthBar.SetMaxHealth(maxHealth);
    }
    
    private void Update()
    {
        if (Vector3.Distance(transform.position, targetPosition) <= proximityRange)
        {
            return;
        }

        intendedMovement = targetPosition - transform.position;
        if (intendedMovement.magnitude > float.Epsilon)
        {
            movementRotation = transform.eulerAngles;
            movementRotation.y = Quaternion.LookRotation(intendedMovement).eulerAngles.y;
            transform.rotation = Quaternion.Euler(movementRotation);  
        }

        sheepAnimator.SetFloat("Speed", Mathf.Max(intendedMovement.magnitude - proximityRange, 0));
        
        characterController.SimpleMove(intendedMovement.normalized * speed);
    }
    
    public void MoveToShepard()
    {
        if (Time.time - latestSheepSoundTime >= minIntervalBetweenSheep)
        {
            latestSheepSoundTime = Time.time;
            audioSource.PlayOneShot(sheepClips.GetRandomItem());
        }
        
        targetPosition = ShepardController.Instance.transform.position;
    }

    public void HealSheep(int healAmount)
    {
        currentHealth = Mathf.Min(currentHealth + healAmount, maxHealth);
    }

    public void DamageSheep(int damageAmount)
    {
        currentHealth = Mathf.Max(currentHealth - damageAmount, 0);
        healthBar.HealthChange(damageAmount, currentHealth);
        if (currentHealth <= 0)
        {
            SheepKilled();   
        }
    }

    private void SheepKilled()
    {
        Destroy(healthBar.gameObject);
        ScreenFlasher.Instance.FlashScreen(10, Color.black);
        StartCoroutine(ReloadScene());
    }

    private IEnumerator ReloadScene()
    {
        yield return new WaitForSeconds(2);
        SceneManager.LoadScene(SceneManager.GetActiveScene().name);
    }

    public void SheepSaved()
    {
        Debug.Log("game won");
        CameraController.Instance.InitiateFinishSequence();
    }
}
