﻿using System;
using System.Collections;
using System.Collections.Generic;
using TMPro;
using UnityEngine;
using UnityEngine.UI;
using UnityUtils;


[RequireComponent(typeof(CharacterController))]
public class ShepardController : MonoBehaviour
{
    [Header("Movement")]
    [SerializeField]
    [Range(5, 20)]
    private float movementSpeed = 10;
    
    [SerializeField]
    private Animator animator;

    [Header("Whistle")]
    [SerializeField]
    [Range(0.1f, 3)]
    private float whistleCooldown = 1;

    [SerializeField] 
    private AudioClip whistleClip;

    [Header("Torch")]
    [SerializeField]
    [Range(0.1f, 5)]
    private float torchCooldown = 3;

    [SerializeField]
    private CooldownBar torchCooldownBar;

    [SerializeField] 
    private TMP_Text torchCountText;

    [SerializeField] 
    private Animator torchAnimator;

    [Header("Axe")]
    [SerializeField] 
    [Range(0.1f, 5)]
    private float logDetectionRange = 3;

    [SerializeField] 
    private LayerMask logDetectionLayer;
    
    [SerializeField]
    [Range(0.5f, 3)]
    private float axeUseDuration = 1;

    [SerializeField] 
    private CooldownBar axeUseBar;

    [SerializeField] 
    private Animator axeAnimator;

    [SerializeField] 
    private AudioClip chopSound;
    
    //input values
    private const string HorizontalAxisName = "Horizontal";
    private const string VerticalAxisName = "Vertical";
    private const string WhistleButtonName = "Whistle";
    private const string TorchButtonName = "Torch";
    private const string AxeButtonName = "Axe";
    private float horizontalAxis;
    private float verticalAxis;
    
    //movement values
    private Vector3 intendedMovement;
    private Vector3 movementRotation;
    private Quaternion cameraRotation;

    //whistle values
    private float latestWhistleUseTime;
    public float RemainingWhistleCooldown => Mathf.Max(whistleCooldown - (Time.time - latestWhistleUseTime), 0);
    public bool IsWhistleOnCooldown => RemainingWhistleCooldown > 0;
    
    //torch values
    private float latestTorchUseTime;
    public float RemainingTorchCooldown => Mathf.Max(torchCooldown - (Time.time - latestTorchUseTime), 0);
    public bool IsTorchOnCooldown => RemainingTorchCooldown > 0;
    public int RemainingTorchCount { get; private set; } = 0;
    public bool CanTorchBeUsed => RemainingTorchCount > 0 && !IsTorchOnCooldown;
    
    //axe values
    public float AxeUseTime { get; private set; }
    private bool isUsingAxe;
    private Log axeTarget;
    
    //cached components
    private CharacterController characterController;
    private AudioSource audioSource;
    
    
    //static instance
    public static ShepardController Instance => instance;
    private static ShepardController instance;
    
    private void Awake()
    {
        if (instance != null && instance != this)
        {
            Destroy(gameObject);
            return;
        }

        instance = this;
        
        characterController = GetComponent<CharacterController>();
        audioSource = GetComponent<AudioSource>();

        cameraRotation = Camera.main.transform.rotation;
    }
    
    private void Update()
    {
        horizontalAxis = Input.GetAxis(HorizontalAxisName);
        verticalAxis = Input.GetAxis(VerticalAxisName);

        if(latestTorchUseTime > 0)
            torchCooldownBar.SetCurrentValue(Time.time - latestTorchUseTime);

        animator.SetFloat("Speed", intendedMovement.magnitude);
        
        if (isUsingAxe)
        {
            intendedMovement = Vector3.zero;
            return;
        }
            
        intendedMovement = cameraRotation * new Vector3(horizontalAxis, 0, verticalAxis) * movementSpeed;

        if (intendedMovement.magnitude > float.Epsilon)
        {
            movementRotation = transform.eulerAngles;
            movementRotation.y = Quaternion.LookRotation(intendedMovement).eulerAngles.y;
            transform.rotation = Quaternion.Euler(movementRotation);    
        }
        
        characterController.SimpleMove(intendedMovement);
        
        if(Input.GetButtonDown(WhistleButtonName))
            UseWhistle();

        if (Input.GetButtonDown(TorchButtonName))
            UseTorch();

        if (Input.GetButtonDown(AxeButtonName))
            UseAxe();
    }

    private void UseWhistle()
    {
        if (IsWhistleOnCooldown)
            return;

        audioSource.PlayOneShot(whistleClip);
        latestWhistleUseTime = Time.time;
        SheepController.Instance.MoveToShepard();
    }

    public void AddTorch()
    {
        RemainingTorchCount++;
        torchCountText.text = RemainingTorchCount.ToString();
    }
    
    private void UseTorch()
    {
        if (!CanTorchBeUsed)
            return;

        RemainingTorchCount--;
        torchCountText.text = RemainingTorchCount.ToString();
        latestTorchUseTime = Time.time;
        StartCoroutine(UseTorchHelper());
    }

    private IEnumerator UseTorchHelper()
    {
        torchAnimator.gameObject.SetActive(true);
        torchAnimator.SetTrigger("Hit");
        yield return new WaitForSeconds(0.5f);
        WolfController.Instance.FleeToRandomPoint();
        yield return new WaitForSeconds(0.5f);
        torchAnimator.gameObject.SetActive(false);
    }

    private void UseAxe()
    {
        if (isUsingAxe)
            return;

        Collider[] foundColliders = Physics.OverlapSphere(transform.position, logDetectionRange, logDetectionLayer, QueryTriggerInteraction.Ignore);
        if (foundColliders.Length == 0)
            return;

        axeTarget = foundColliders[0].GetComponent<Log>();
        if (axeTarget == null)
            return;
        
        isUsingAxe = true;
        StartCoroutine(UseAxeHelper());
    }

    private IEnumerator UseAxeHelper()
    {
        AxeUseTime = 0;
        
        Vector3 turnRotation = transform.eulerAngles;
        turnRotation.y = Quaternion.LookRotation(axeTarget.transform.position - transform.position).eulerAngles.y;
        transform.rotation = Quaternion.Euler(turnRotation);    
        //transform.rotation = Quaternion.LookRotation(axeTarget.transform.position - transform.position);
        axeAnimator.gameObject.SetActive(true);
        
        while (AxeUseTime < axeUseDuration)
        {
            if(AxeUseTime > 0.1f)
                axeAnimator.SetTrigger("Blow");
            
            axeUseBar.SetCurrentValue(AxeUseTime);
            AxeUseTime += Time.deltaTime;
            yield return null;
        }
        
        audioSource.PlayOneShot(chopSound);
        axeTarget.AxeUsed();
        isUsingAxe = false;
        axeTarget = null;
        axeAnimator.gameObject.SetActive(false);

        //CameraShaker.Instance.ShakeOnce(1, 1);
        
        yield return new WaitForSeconds(0.1f);
        axeUseBar.SetCurrentValue(0);
    }
}
