﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AI;
using UnityUtils;

[RequireComponent(typeof(NavMeshAgent))]
[RequireComponent(typeof(Animator))]
public class WolfController : MonoBehaviour
{
    [SerializeField] 
    [Range(1, 10)]
    private float attackRange = 5;

    [SerializeField]
    [Range(1, 5)]
    private int attackDamage = 1;

    [SerializeField]
    [Range(0.01f, 3)]
    private float waitTimeAfterAttack = 1;

    [SerializeField] 
    private float randomPointRange = 50;

    [SerializeField] 
    private LayerMask terrainLayer;

    [SerializeField] 
    [Range(1, 5)]
    private float chaseSpeed = 3.5f;
    
    [SerializeField]
    [Range(5, 15)]
    private float fleeSpeed = 10;

    [SerializeField] 
    private Animator wolfAnimator;

    [SerializeField] 
    private AudioClip[] attackClips;
    
    private bool isChasing;
    private bool isFleeing;
    private Vector3 fleePoint;
    
    private NavMeshAgent navMeshAgent;
    private AudioSource audioSource;
    
    public static WolfController Instance => instance;
    private static WolfController instance;
    
    private void Awake()
    {
        if (instance != null && instance != this)
        {
            Destroy(gameObject);
            return;
        }

        instance = this;

        navMeshAgent = GetComponent<NavMeshAgent>();
        audioSource = GetComponent<AudioSource>();
        
        TeleportWolf(transform.position);
    }

    private void Update()
    {
        if (isChasing)
        {
            navMeshAgent.SetDestination(SheepController.Instance.transform.position);
            if (Vector3.Distance(SheepController.Instance.transform.position, transform.position) <= attackRange)
            {
                SheepController.Instance.DamageSheep(attackDamage);
                StopChasing();
                StartCoroutine(FleeAfterAttack());
            }
        }
        else if (isFleeing)
        {
            navMeshAgent.SetDestination(fleePoint);
        }
        
        wolfAnimator.SetFloat("Speed", navMeshAgent.velocity.magnitude);
    }

    public void TeleportWolf(Vector3 point)
    {
        if (isChasing)
            return;
        
        navMeshAgent.Warp(point);
        transform.position = point;
    }

    public void StartChasing()
    {
        LevelManager.Instance.LatestWolfAttackTime = Time.time;
        isChasing = true;
        navMeshAgent.speed = chaseSpeed;
        audioSource.PlayOneShot(attackClips.GetRandomItem());
    }

    public void StopChasing()
    {
        isChasing = false;
    }

    public void FleeToRandomPoint()
    {
        Vector3 randomPoint;
        RaycastHit hitInfo;
        int tryCount = 0;
        
        do
        {
            Vector2 randomSpherePoint = Random.insideUnitCircle * randomPointRange;
            randomPoint = new Vector3(randomSpherePoint.x, transform.position.y, randomSpherePoint.y);
            Physics.Raycast(randomPoint + Vector3.up * 50, Vector3.down, out hitInfo, 100, terrainLayer);

            tryCount++;
        } while (hitInfo.collider == null && tryCount < 100);
        
        FleeToPoint(randomPoint);
    }
    
    public void FleeToPoint(Vector3 point)
    {
        StopChasing();
        navMeshAgent.speed = fleeSpeed;
        isFleeing = true;
        fleePoint = point;
    }

    private IEnumerator FleeAfterAttack()
    {
        yield return new WaitForSeconds(waitTimeAfterAttack);
        FleeToRandomPoint();
    }
}
